@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -30,9 +30,9 @@ from botorch.acquisition.knowledge_gradient import (
 )
 from botorch.acquisition.max_value_entropy_search import (
     MaxValueBase,
+    qLowerBoundMaxValueEntropy,
     qMaxValueEntropy,
     qMultiFidelityMaxValueEntropy,
-    qLowerBoundMaxValueEntropy,
 )
 from botorch.acquisition.monte_carlo import (
     MCAcquisitionFunction,
@@ -50,6 +50,7 @@ from botorch.acquisition.objective import (
     LinearMCObjective,
     MCAcquisitionObjective,
     ScalarizedObjective,
+    ScalarizedPosteriorTransform,
 )
 from botorch.acquisition.proximal import ProximalAcquisitionFunction
 from botorch.acquisition.utils import get_acquisition_function
@@ -88,6 +89,7 @@ __all__ = [
     "MCAcquisitionFunction",
     "MCAcquisitionObjective",
     "ScalarizedObjective",
+    "ScalarizedPosteriorTransform",
     "get_acquisition_function",
     "get_acqf_input_constructor",
 ]
