@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,10 +12,11 @@ from __future__ import annotations
 
 import warnings
 from abc import ABC, abstractmethod
-from typing import Optional
+from typing import Optional, Callable
 
-from botorch.exceptions import BotorchWarning
+from botorch.exceptions import BotorchWarning, UnsupportedError
 from botorch.models.model import Model
+from botorch.posteriors.posterior import Posterior
 from torch import Tensor
 from torch.nn import Module
 
@@ -32,6 +33,35 @@ class AcquisitionFunction(Module, ABC):
         super().__init__()
         self.add_module("model", model)
 
+    @classmethod
+    def _deprecate_acqf_objective(
+        cls,
+        posterior_transform: Optional[Callable[[Posterior], Posterior]],
+        objective: Optional[Module],
+    ) -> Optional[Callable[[Posterior], Posterior]]:
+        from botorch.acquisition.objective import (
+            ScalarizedObjective,
+            ScalarizedPosteriorTransform,
+        )
+
+        if objective is None:
+            return posterior_transform
+        warnings.warn(
+            f"{cls.__name__} got a non-MC `objective`. The non-MC "
+            "AcquisitionObjectives and the `objective` argument to"
+            "AnalyticAcquisitionFunctions are DEPRECATED and will be removed in the"
+            "next version. Use `posterior_transform` instead.",
+            DeprecationWarning,
+        )
+        if not isinstance(objective, ScalarizedObjective):
+            raise UnsupportedError(
+                f"{cls.__name__} only supports ScalarizedObjective "
+                "(DEPRECATED) type objectives."
+            )
+        return ScalarizedPosteriorTransform(
+            weights=objective.weights, offset=objective.offset
+        )
+
     def set_X_pending(self, X_pending: Optional[Tensor] = None) -> None:
         r"""Informs the acquisition function about pending design points.
 
