@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,17 +9,30 @@ A registry of helpers for generating inputs to acquisition function
 constructors programmatically from a consistent input format.
 """
 
-from typing import Any, Callable, Dict, Optional, Tuple, Type, Union
+from __future__ import annotations
+
+import warnings
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 import torch
 from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.analytic import (
+    ConstrainedExpectedImprovement,
     ExpectedImprovement,
+    NoisyExpectedImprovement,
     PosteriorMean,
     ProbabilityOfImprovement,
     UpperConfidenceBound,
-    ConstrainedExpectedImprovement,
-    NoisyExpectedImprovement,
+)
+from botorch.acquisition.cost_aware import InverseCostWeightedUtility
+from botorch.acquisition.fixed_feature import FixedFeatureAcquisitionFunction
+from botorch.acquisition.knowledge_gradient import (
+    qKnowledgeGradient,
+    qMultiFidelityKnowledgeGradient,
+)
+from botorch.acquisition.max_value_entropy_search import (
+    qMaxValueEntropy,
+    qMultiFidelityMaxValueEntropy,
 )
 from botorch.acquisition.monte_carlo import (
     qExpectedImprovement,
@@ -30,10 +43,12 @@ from botorch.acquisition.monte_carlo import (
 )
 from botorch.acquisition.multi_objective import (
     ExpectedHypervolumeImprovement,
+    MCMultiOutputObjective,
     qExpectedHypervolumeImprovement,
     qNoisyExpectedHypervolumeImprovement,
 )
 from botorch.acquisition.multi_objective.objective import (
+    AnalyticMultiOutputObjective,
     IdentityAnalyticMultiOutputObjective,
     IdentityMCMultiOutputObjective,
 )
@@ -41,10 +56,19 @@ from botorch.acquisition.multi_objective.utils import get_default_partitioning_a
 from botorch.acquisition.objective import (
     AcquisitionObjective,
     IdentityMCObjective,
+    MCAcquisitionObjective,
     ScalarizedObjective,
+    PosteriorTransform,
+    ScalarizedPosteriorTransform,
+)
+from botorch.acquisition.utils import (
+    expand_trace_observations,
+    project_to_target_fidelity,
 )
 from botorch.exceptions.errors import UnsupportedError
+from botorch.models.cost import AffineFidelityCostModel
 from botorch.models.model import Model
+from botorch.optim.optimize import optimize_acqf
 from botorch.sampling.samplers import IIDNormalSampler, MCSampler, SobolQMCNormalSampler
 from botorch.utils.constraints import get_outcome_constraint_transforms
 from botorch.utils.containers import TrainingData
@@ -114,6 +138,40 @@ def _register_acqf_input_constructor(
     ACQF_INPUT_CONSTRUCTOR_REGISTRY[acqf_cls] = input_constructor
 
 
+# ------------------------- Deprecation Helpers ------------------------- #
+
+
+def _deprecate_objective_arg(
+    posterior_transform: Optional[PosteriorTransform] = None,
+    objective: Optional[AcquisitionObjective] = None,
+) -> Optional[PosteriorTransform]:
+    if posterior_transform is not None:
+        if objective is None:
+            return posterior_transform
+        else:
+            raise RuntimeError(
+                "Got both a non-MC objective (DEPRECATED) and a posterior "
+                "transform. Use only a posterior transform instead."
+            )
+    elif objective is not None:
+        warnings.warn(
+            "The `objective` argument to AnalyticAcquisitionFunctions is deprecated "
+            "and will be removed in the next version. Use `posterior_transform` "
+            "instead.",
+            DeprecationWarning,
+        )
+        if not isinstance(objective, ScalarizedObjective):
+            raise UnsupportedError(
+                "Analytic acquisition functions only support ScalarizedObjective "
+                "(DEPRECATED) type objectives."
+            )
+        return ScalarizedPosteriorTransform(
+            weights=objective.weights, offset=objective.offset
+        )
+    else:
+        return None
+
+
 # --------------------- Input argument constructors --------------------- #
 
 
@@ -121,7 +179,7 @@ def _register_acqf_input_constructor(
 def construct_inputs_analytic_base(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     **kwargs: Any,
 ) -> Dict[str, Any]:
     r"""Construct kwargs for basic analytic acquisition functions.
@@ -130,19 +188,26 @@ def construct_inputs_analytic_base(
         model: The model to be used in the acquisition function.
         training_data: A TrainingData object contraining the model's
             training data. `best_f` is extracted from here.
-        objective: The objective to in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
 
     Returns:
         A dict mapping kwarg names of the constructor to values.
     """
-    return {"model": model, "objective": objective}
+    return {
+        "model": model,
+        "posterior_transform": _deprecate_objective_arg(
+            posterior_transform=posterior_transform,
+            objective=kwargs.get("objective"),
+        ),
+    }
 
 
 @acqf_input_constructor(ExpectedImprovement, ProbabilityOfImprovement)
 def construct_inputs_best_f(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     maximize: bool = True,
     **kwargs: Any,
 ) -> Dict[str, Any]:
@@ -152,17 +217,26 @@ def construct_inputs_best_f(
         model: The model to be used in the acquisition function.
         training_data: A TrainingData object contraining the model's
             training data. `best_f` is extracted from here.
-        objective: The objective to in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
         maximize: If True, consider the problem a maximization problem.
 
     Returns:
         A dict mapping kwarg names of the constructor to values.
     """
     base_inputs = construct_inputs_analytic_base(
-        model=model, training_data=training_data, objective=objective
+        model=model,
+        training_data=training_data,
+        posterior_transform=posterior_transform,
+        **kwargs,
     )
     best_f = kwargs.get(
-        "best_f", get_best_f_analytic(training_data=training_data, objective=objective)
+        "best_f",
+        get_best_f_analytic(
+            training_data=training_data,
+            objective=kwargs.get("objective"),
+            posterior_transform=posterior_transform,
+        ),
     )
     return {**base_inputs, "best_f": best_f, "maximize": maximize}
 
@@ -171,7 +245,7 @@ def construct_inputs_best_f(
 def construct_inputs_ucb(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     beta: Union[float, Tensor] = 0.2,
     maximize: bool = True,
     **kwargs: Any,
@@ -182,7 +256,8 @@ def construct_inputs_ucb(
         model: The model to be used in the acquisition function.
         training_data: A TrainingData object contraining the model's
             training data. `best_f` is extracted from here.
-        objective: The objective to in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
         beta: Either a scalar or a one-dim tensor with `b` elements (batch mode)
             representing the trade-off parameter between mean and covariance
         maximize: If True, consider the problem a maximization problem.
@@ -191,7 +266,10 @@ def construct_inputs_ucb(
         A dict mapping kwarg names of the constructor to values.
     """
     base_inputs = construct_inputs_analytic_base(
-        model=model, training_data=training_data, objective=objective
+        model=model,
+        training_data=training_data,
+        posterior_transform=posterior_transform,
+        **kwargs,
     )
     return {**base_inputs, "beta": beta, "maximize": maximize}
 
@@ -269,7 +347,8 @@ def construct_inputs_noisy_ei(
 def construct_inputs_mc_base(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     X_pending: Optional[Tensor] = None,
     sampler: Optional[MCSampler] = None,
     **kwargs: Any,
@@ -281,7 +360,9 @@ def construct_inputs_mc_base(
         training_data: A TrainingData object contraining the model's
             training data. Used e.g. to extract inputs such as `best_f`
             for expected improvement acquisition functions.
-        objective: The objective to in the acquisition function.
+        objective: The objective to be used in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
         X_pending: A `batch_shape, m x d`-dim Tensor of `m` design points
             that have points that have been submitted for function evaluation
             but have not yet been evaluated.
@@ -294,6 +375,7 @@ def construct_inputs_mc_base(
     return {
         "model": model,
         "objective": objective,
+        "posterior_transform": posterior_transform,
         "X_pending": X_pending,
         "sampler": sampler,
     }
@@ -303,7 +385,8 @@ def construct_inputs_mc_base(
 def construct_inputs_qEI(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     X_pending: Optional[Tensor] = None,
     sampler: Optional[MCSampler] = None,
     **kwargs: Any,
@@ -315,7 +398,9 @@ def construct_inputs_qEI(
         training_data: A TrainingData object contraining the model's
             training data. Used e.g. to extract inputs such as `best_f`
             for expected improvement acquisition functions.
-        objective: The objective to in the acquisition function.
+        objective: The objective to be used in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
         X_pending: A `m x d`-dim Tensor of `m` design points that have been
             submitted for function evaluation but have not yet been evaluated.
             Concatenated into X upon forward call.
@@ -329,13 +414,19 @@ def construct_inputs_qEI(
         model=model,
         training_data=training_data,
         objective=objective,
+        posterior_transform=posterior_transform,
         sampler=sampler,
         X_pending=X_pending,
     )
     # TODO: Dedup handling of this here and in the constructor (maybe via a
     # shared classmethod doing this)
     best_f = kwargs.get(
-        "best_f", get_best_f_mc(training_data=training_data, objective=objective)
+        "best_f",
+        get_best_f_mc(
+            training_data=training_data,
+            objective=objective,
+            posterior_transform=posterior_transform,
+        ),
     )
     return {**base_inputs, "best_f": best_f}
 
@@ -344,7 +435,8 @@ def construct_inputs_qEI(
 def construct_inputs_qNEI(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     X_pending: Optional[Tensor] = None,
     sampler: Optional[MCSampler] = None,
     X_baseline: Optional[Tensor] = None,
@@ -359,7 +451,9 @@ def construct_inputs_qNEI(
             training data. Used e.g. to extract inputs such as `best_f`
             for expected improvement acquisition functions. Only block-
             design training data currently supported.
-        objective: The objective to in the acquisition function.
+        objective: The objective to be used in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
         X_pending: A `m x d`-dim Tensor of `m` design points that have been
             submitted for function evaluation but have not yet been evaluated.
             Concatenated into X upon forward call.
@@ -379,6 +473,7 @@ def construct_inputs_qNEI(
         model=model,
         training_data=training_data,
         objective=objective,
+        posterior_transform=posterior_transform,
         sampler=sampler,
         X_pending=X_pending,
     )
@@ -397,7 +492,8 @@ def construct_inputs_qNEI(
 def construct_inputs_qPI(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     X_pending: Optional[Tensor] = None,
     sampler: Optional[MCSampler] = None,
     tau: float = 1e-3,
@@ -411,7 +507,9 @@ def construct_inputs_qPI(
         training_data: A TrainingData object contraining the model's
             training data. Used e.g. to extract inputs such as `best_f`
             for expected improvement acquisition functions.
-        objective: The objective to in the acquisition function.
+        objective: The objective to be used in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
         X_pending: A `m x d`-dim Tensor of `m` design points that have been
             submitted for function evaluation but have not yet been evaluated.
             Concatenated into X upon forward call.
@@ -431,6 +529,7 @@ def construct_inputs_qPI(
         model=model,
         training_data=training_data,
         objective=objective,
+        posterior_transform=posterior_transform,
         sampler=sampler,
         X_pending=X_pending,
     )
@@ -449,7 +548,8 @@ def construct_inputs_qPI(
 def construct_inputs_qUCB(
     model: Model,
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
     X_pending: Optional[Tensor] = None,
     sampler: Optional[MCSampler] = None,
     beta: float = 0.2,
@@ -462,7 +562,9 @@ def construct_inputs_qUCB(
         training_data: A TrainingData object contraining the model's
             training data. Used e.g. to extract inputs such as `best_f`
             for expected improvement acquisition functions.
-        objective: The objective to in the acquisition function.
+        objective: The objective to be used in the acquisition function.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
         X_pending: A `m x d`-dim Tensor of `m` design points that have been
             submitted for function evaluation but have not yet been evaluated.
             Concatenated into X upon forward call.
@@ -477,6 +579,7 @@ def construct_inputs_qUCB(
         model=model,
         training_data=training_data,
         objective=objective,
+        posterior_transform=posterior_transform,
         sampler=sampler,
         X_pending=X_pending,
     )
@@ -497,7 +600,7 @@ def construct_inputs_EHVI(
     model: Model,
     training_data: TrainingData,
     objective_thresholds: Tensor,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[AnalyticMultiOutputObjective] = None,
     **kwargs: Any,
 ) -> Dict[str, Any]:
     r"""Construct kwargs for `ExpectedHypervolumeImprovement` constructor."""
@@ -547,7 +650,7 @@ def construct_inputs_qEHVI(
     model: Model,
     training_data: TrainingData,
     objective_thresholds: Tensor,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCMultiOutputObjective] = None,
     **kwargs: Any,
 ) -> Dict[str, Any]:
     r"""Construct kwargs for `qExpectedHypervolumeImprovement` constructor."""
@@ -601,7 +704,7 @@ def construct_inputs_qNEHVI(
     model: Model,
     training_data: TrainingData,
     objective_thresholds: Tensor,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCMultiOutputObjective] = None,
     **kwargs: Any,
 ) -> Dict[str, Any]:
     r"""Construct kwargs for `qNoisyExpectedHypervolumeImprovement` constructor."""
@@ -639,15 +742,212 @@ def construct_inputs_qNEHVI(
     }
 
 
+@acqf_input_constructor(qMaxValueEntropy)
+def construct_inputs_qMES(
+    model: Model,
+    training_data: TrainingData,
+    bounds: List[Tuple[float, float]],
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
+    candidate_size: int = 1000,
+    **kwargs: Any,
+) -> Dict[str, Any]:
+    r"""Construct kwargs for `qMaxValueEntropy` constructor."""
+    inputs_mc = construct_inputs_mc_base(
+        model=model,
+        training_data=training_data,
+        objective=objective,
+        **kwargs,
+    )
+
+    _kw = {"dtype": training_data.X.dtype, "device": training_data.X.device}
+    _rvs = torch.rand(candidate_size, len(bounds), **_kw)
+    _bounds = torch.tensor(bounds, **_kw).transpose(0, 1)
+    return {
+        **inputs_mc,
+        "candidate_set": _bounds[0] + (_bounds[1] - _bounds[0]) * _rvs,
+        "maximize": kwargs.get("maximize", True),
+    }
+
+
+def construct_inputs_mf_base(
+    model: Model,
+    training_data: TrainingData,
+    target_fidelities: Dict[int, Union[int, float]],
+    fidelity_weights: Optional[Dict[int, float]] = None,
+    cost_intercept: float = 1.0,
+    num_trace_observations: int = 0,
+    **ignore: Any,
+) -> Dict[str, Any]:
+    r"""Construct kwargs for a multifidetlity acquisition function's constructor."""
+    if fidelity_weights is None:
+        fidelity_weights = {f: 1.0 for f in target_fidelities}
+
+    if set(target_fidelities) != set(fidelity_weights):
+        raise RuntimeError(
+            "Must provide the same indices for target_fidelities "
+            f"({set(target_fidelities)}) and fidelity_weights "
+            f" ({set(fidelity_weights)})."
+        )
+
+    cost_aware_utility = InverseCostWeightedUtility(
+        cost_model=AffineFidelityCostModel(
+            fidelity_weights=fidelity_weights, fixed_cost=cost_intercept
+        )
+    )
+
+    return {
+        "target_fidelities": target_fidelities,
+        "cost_aware_utility": cost_aware_utility,
+        "expand": lambda X: expand_trace_observations(
+            X=X,
+            fidelity_dims=sorted(target_fidelities),
+            num_trace_obs=num_trace_observations,
+        ),
+        "project": lambda X: project_to_target_fidelity(
+            X=X, target_fidelities=target_fidelities
+        ),
+    }
+
+
+@acqf_input_constructor(qKnowledgeGradient)
+def construct_inputs_qKG(
+    model: Model,
+    training_data: TrainingData,
+    bounds: List[Tuple[float, float]],
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
+    target_fidelities: Optional[Dict[int, float]] = None,
+    num_fantasies: int = 64,
+    **kwargs: Any,
+) -> Dict[str, Any]:
+    r"""Construct kwargs for `qKnowledgeGradient` constructor."""
+
+    inputs_mc = construct_inputs_mc_base(
+        model=model,
+        training_data=training_data,
+        objective=objective,
+        posterior_transform=posterior_transform,
+        **kwargs,
+    )
+
+    _bounds = torch.tensor(
+        data=bounds,
+        dtype=training_data.X.dtype,
+        device=training_data.X.device,
+    )
+
+    _, current_value = optimize_objective(
+        model=model,
+        bounds=_bounds.t(),
+        q=1,
+        target_fidelities=target_fidelities,
+        objective=objective,
+        posterior_transform=posterior_transform,
+        **kwargs,
+    )
+
+    return {
+        **inputs_mc,
+        "num_fantasies": num_fantasies,
+        "current_value": current_value.detach().cpu().max(),
+    }
+
+
+@acqf_input_constructor(qMultiFidelityKnowledgeGradient)
+def construct_inputs_qMFKG(
+    model: Model,
+    training_data: TrainingData,
+    bounds: List[Tuple[float, float]],
+    target_fidelities: Dict[int, Union[int, float]],
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
+    **kwargs: Any,
+) -> Dict[str, Any]:
+    r"""Construct kwargs for `qMultiFidelityKnowledgeGradient` constructor."""
+
+    inputs_mf = construct_inputs_mf_base(
+        model=model,
+        training_data=training_data,
+        target_fidelities=target_fidelities,
+        **kwargs,
+    )
+
+    inputs_kg = construct_inputs_qKG(
+        model=model,
+        training_data=training_data,
+        bounds=bounds,
+        objective=objective,
+        posterior_transform=posterior_transform,
+        **kwargs,
+    )
+
+    return {**inputs_mf, **inputs_kg}
+
+
+@acqf_input_constructor(qMultiFidelityMaxValueEntropy)
+def construct_inputs_qMFMES(
+    model: Model,
+    training_data: TrainingData,
+    bounds: List[Tuple[float, float]],
+    target_fidelities: Dict[int, Union[int, float]],
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
+    **kwargs: Any,
+) -> Dict[str, Any]:
+    r"""Construct kwargs for `qMultiFidelityMaxValueEntropy` constructor."""
+    inputs_mf = construct_inputs_mf_base(
+        model=model,
+        training_data=training_data,
+        target_fidelities=target_fidelities,
+        **kwargs,
+    )
+
+    inputs_qmes = construct_inputs_qMES(
+        model=model,
+        training_data=training_data,
+        bounds=bounds,
+        objective=objective,
+        posterior_transform=posterior_transform,
+        **kwargs,
+    )
+
+    _bounds = torch.tensor(
+        data=bounds,
+        dtype=training_data.X.dtype,
+        device=training_data.X.device,
+    )
+
+    _, current_value = optimize_objective(
+        model=model,
+        bounds=_bounds.t(),
+        q=1,
+        objective=objective,
+        posterior_transform=posterior_transform,
+        target_fidelities=target_fidelities,
+        **kwargs,
+    )
+
+    return {
+        **inputs_mf,
+        **inputs_qmes,
+        "current_value": current_value.detach().cpu().max(),
+    }
+
+
 def get_best_f_analytic(
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
+    **kwargs,
 ) -> Tensor:
     if not training_data.is_block_design:
         raise NotImplementedError("Currently only block designs are supported.")
     Y = training_data.Y
-    if isinstance(objective, ScalarizedObjective):
-        return objective.evaluate(Y).max(-1).values
+    posterior_transform = _deprecate_objective_arg(
+        posterior_transform=posterior_transform, objective=kwargs.get("objective", None)
+    )
+    if posterior_transform is not None:
+        return posterior_transform.evaluate(Y).max(-1).values
     if Y.shape[-1] > 1:
         raise NotImplementedError(
             "Analytic acquisition functions currently only work with "
@@ -658,11 +958,25 @@ def get_best_f_analytic(
 
 def get_best_f_mc(
     training_data: TrainingData,
-    objective: Optional[AcquisitionObjective] = None,
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
 ) -> Tensor:
     if not training_data.is_block_design:
         raise NotImplementedError("Currently only block designs are supported.")
     Y = training_data.Y
+    posterior_transform = _deprecate_objective_arg(
+        posterior_transform=posterior_transform,
+        objective=objective
+        if not isinstance(objective, MCAcquisitionObjective)
+        else None,
+    )
+    if posterior_transform is not None:
+        # retain the original tensor dimension since objective expects explicit
+        # output dimension.
+        Y_dim = Y.dim()
+        Y = posterior_transform.evaluate(Y)
+        if Y.dim() < Y_dim:
+            Y = Y.unsqueeze(-1)
     if objective is None:
         if Y.shape[-1] > 1:
             raise UnsupportedError(
@@ -671,4 +985,112 @@ def get_best_f_mc(
                 "acquisition functions)."
             )
         objective = IdentityMCObjective()
-    return objective(training_data.Y).max(-1).values
+    return objective(Y).max(-1).values
+
+
+def optimize_objective(
+    model: Model,
+    bounds: Tensor,
+    q: int,
+    objective: Optional[MCAcquisitionObjective] = None,
+    posterior_transform: Optional[PosteriorTransform] = None,
+    linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
+    fixed_features: Optional[Dict[int, float]] = None,
+    target_fidelities: Optional[Dict[int, float]] = None,
+    qmc: bool = True,
+    mc_samples: int = 512,
+    seed_inner: Optional[int] = None,
+    optimizer_options: Dict[str, Any] = None,
+    post_processing_func: Optional[Callable[[Tensor], Tensor]] = None,
+    batch_initial_conditions: Optional[Tensor] = None,
+    sequential: bool = False,
+    **ignore,
+) -> Tuple[Tensor, Tensor]:
+    r"""Optimize an objective under the given model.
+
+    Args:
+        model: The model to be used in the objective.
+        bounds: A `2 x d` tensor of lower and upper bounds for each column of `X`.
+        q: The cardinality of input sets on which the objective is to be evaluated.
+        objective: The objective to optimize.
+        posterior_transform: The posterior transform to be used in the
+            acquisition function.
+        linear_constraints: A tuple of (A, b). Given `k` linear constraints on a
+            `d`-dimensional space, `A` is `k x d` and `b` is `k x 1` such that
+            `A x <= b`. (Not used by single task models).
+        fixed_features: A dictionary of feature assignments `{feature_index: value}` to
+            hold fixed during generation.
+        target_fidelities: A dictionary mapping input feature indices to fidelity
+            values. Defaults to `{-1: 1.0}`.
+        qmc: Toggle for enabling (qmc=1) or disabling (qmc=0) use of Quasi Monte Carlo.
+        mc_samples: Integer number of samples used to estimate Monte Carlo objectives.
+        seed_inner: Integer seed used to initialize the sampler passed to MCObjective.
+        optimizer_options: Table used to lookup keyword arguments for the optimizer.
+        post_processing_func: A function that post-processes an optimization
+            result appropriately (i.e. according to `round-trip` transformations).
+        batch_initial_conditions: A Tensor of initial values for the optimizer.
+        sequential: If False, uses joint optimization, otherwise uses sequential
+            optimization.
+
+    Returns:
+        A tuple containing the best input locations and corresponding objective values.
+    """
+    if optimizer_options is None:
+        optimizer_options = {}
+
+    if objective is not None:
+        sampler_cls = SobolQMCNormalSampler if qmc else IIDNormalSampler
+        acq_function = qSimpleRegret(
+            model=model,
+            objective=objective,
+            posterior_transform=posterior_transform,
+            sampler=sampler_cls(num_samples=mc_samples, seed=seed_inner),
+        )
+    else:
+        acq_function = PosteriorMean(
+            model=model, posterior_transform=posterior_transform
+        )
+
+    if fixed_features:
+        acq_function = FixedFeatureAcquisitionFunction(
+            acq_function=acq_function,
+            d=bounds.shape[-1],
+            columns=list(fixed_features.keys()),
+            values=list(fixed_features.values()),
+        )
+        free_feature_dims = list(range(len(bounds)) - fixed_features.keys())
+        free_feature_bounds = bounds[:, free_feature_dims]  # (2, d' <= d)
+    else:
+        free_feature_bounds = bounds
+
+    if linear_constraints is None:
+        inequality_constraints = None
+    else:
+        A, b = linear_constraints
+        inequality_constraints = []
+        k, d = A.shape
+        for i in range(k):
+            indicies = A[i, :].nonzero(as_tuple=False).squeeze()
+            coefficients = -A[i, indicies]
+            rhs = -b[i, 0]
+            inequality_constraints.append((indicies, coefficients, rhs))
+
+    return optimize_acqf(
+        acq_function=acq_function,
+        bounds=free_feature_bounds,
+        q=q,
+        num_restarts=optimizer_options.get("num_restarts", 60),
+        raw_samples=optimizer_options.get("raw_samples", 1024),
+        options={
+            "batch_limit": optimizer_options.get("batch_limit", 8),
+            "maxiter": optimizer_options.get("maxiter", 200),
+            "nonnegative": optimizer_options.get("nonnegative", False),
+            "method": optimizer_options.get("method", "L-BFGS-B"),
+        },
+        inequality_constraints=inequality_constraints,
+        fixed_features=None,  # handled inside the acquisition function
+        post_processing_func=post_processing_func,
+        batch_initial_conditions=batch_initial_conditions,
+        return_best_only=True,
+        sequential=sequential,
+    )
