@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -22,14 +22,21 @@ from __future__ import annotations
 
 import math
 from abc import ABC, abstractmethod
+from copy import deepcopy
 from typing import Any, Optional, Union
 
 import torch
 from botorch.acquisition.acquisition import AcquisitionFunction
-from botorch.acquisition.objective import IdentityMCObjective, MCAcquisitionObjective
+from botorch.acquisition.cached_cholesky import CachedCholeskyMCAcquisitionFunction
+from botorch.acquisition.objective import (
+    IdentityMCObjective,
+    MCAcquisitionObjective,
+    PosteriorTransform,
+)
 from botorch.acquisition.utils import prune_inferior_points
 from botorch.exceptions.errors import UnsupportedError
 from botorch.models.model import Model
+from botorch.posteriors.posterior import Posterior
 from botorch.sampling.samplers import MCSampler, SobolQMCNormalSampler
 from botorch.utils.transforms import (
     concatenate_pending_points,
@@ -47,6 +54,7 @@ class MCAcquisitionFunction(AcquisitionFunction, ABC):
         model: Model,
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCAcquisitionObjective] = None,
+        posterior_transform: Optional[PosteriorTransform] = None,
         X_pending: Optional[Tensor] = None,
     ) -> None:
         r"""Constructor for the MCAcquisitionFunction base class.
@@ -57,6 +65,7 @@ class MCAcquisitionFunction(AcquisitionFunction, ABC):
                 `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`.
             objective: The MCAcquisitionObjective under which the samples are
                 evaluated. Defaults to `IdentityMCObjective()`.
+            posterior_transform: A PosteriorTransform (optional).
             X_pending: A `batch_shape, m x d`-dim Tensor of `m` design points
                 that have points that have been submitted for function evaluation
                 but have not yet been evaluated.
@@ -65,17 +74,20 @@ class MCAcquisitionFunction(AcquisitionFunction, ABC):
         if sampler is None:
             sampler = SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)
         self.add_module("sampler", sampler)
-        if objective is None:
-            if model.num_outputs != 1:
+        if objective is None and model.num_outputs != 1:
+            if posterior_transform is None:
                 raise UnsupportedError(
-                    "Must specify an objective when using a multi-output model."
+                    "Must specify an objective or a posterior transform when using "
+                    "a multi-output model."
                 )
+            elif not posterior_transform.scalarize:
+                raise UnsupportedError(
+                    "If using a multi-output model without an objective, "
+                    "posterior_transform must scalarize the output."
+                )
+        if objective is None:
             objective = IdentityMCObjective()
-        elif not isinstance(objective, MCAcquisitionObjective):
-            raise UnsupportedError(
-                "Only objectives of type MCAcquisitionObjective are supported for "
-                "MC acquisition functions."
-            )
+        self.posterior_transform = posterior_transform
         self.add_module("objective", objective)
         self.set_X_pending(X_pending)
 
@@ -115,6 +127,7 @@ class qExpectedImprovement(MCAcquisitionFunction):
         best_f: Union[float, Tensor],
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCAcquisitionObjective] = None,
+        posterior_transform: Optional[PosteriorTransform] = None,
         X_pending: Optional[Tensor] = None,
         **kwargs: Any,
     ) -> None:
@@ -129,13 +142,18 @@ class qExpectedImprovement(MCAcquisitionFunction):
                 `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`
             objective: The MCAcquisitionObjective under which the samples are evaluated.
                 Defaults to `IdentityMCObjective()`.
+            posterior_transform: A PosteriorTransform (optional).
             X_pending:  A `m x d`-dim Tensor of `m` design points that have been
                 submitted for function evaluation but have not yet been evaluated.
                 Concatenated into X upon forward call. Copied and set to have no
                 gradient.
         """
         super().__init__(
-            model=model, sampler=sampler, objective=objective, X_pending=X_pending
+            model=model,
+            sampler=sampler,
+            objective=objective,
+            posterior_transform=posterior_transform,
+            X_pending=X_pending,
         )
         self.register_buffer("best_f", torch.as_tensor(best_f, dtype=float))
 
@@ -153,7 +171,9 @@ class qExpectedImprovement(MCAcquisitionFunction):
             design points `X`, where `batch_shape'` is the broadcasted batch shape of
             model and input `X`.
         """
-        posterior = self.model.posterior(X)
+        posterior = self.model.posterior(
+            X=X, posterior_transform=self.posterior_transform
+        )
         samples = self.sampler(posterior)
         obj = self.objective(samples, X=X)
         obj = (obj - self.best_f.unsqueeze(-1).to(obj)).clamp_min(0)
@@ -161,7 +181,9 @@ class qExpectedImprovement(MCAcquisitionFunction):
         return q_ei
 
 
-class qNoisyExpectedImprovement(MCAcquisitionFunction):
+class qNoisyExpectedImprovement(
+    MCAcquisitionFunction, CachedCholeskyMCAcquisitionFunction
+):
     r"""MC-based batch Noisy Expected Improvement.
 
     This function does not assume a `best_f` is known (which would require
@@ -185,8 +207,10 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
         X_baseline: Tensor,
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCAcquisitionObjective] = None,
+        posterior_transform: Optional[PosteriorTransform] = None,
         X_pending: Optional[Tensor] = None,
         prune_baseline: bool = False,
+        cache_root: bool = True,
         **kwargs: Any,
     ) -> None:
         r"""q-Noisy Expected Improvement.
@@ -200,6 +224,7 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
                 `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`.
             objective: The MCAcquisitionObjective under which the samples are
                 evaluated. Defaults to `IdentityMCObjective()`.
+            posterior_transform: A PosteriorTransform (optional).
             X_pending: A `batch_shape x m x d`-dim Tensor of `m` design points
                 that have points that have been submitted for function evaluation
                 but have not yet been evaluated. Concatenated into `X` upon
@@ -210,19 +235,119 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
                 customize pruning parameters, instead manually call
                 `botorch.acquisition.utils.prune_inferior_points` on `X_baseline`
                 before instantiating the acquisition function.
+            cache_root: A boolean indicating whether to cache the root
+                decomposition over `X_baseline` and use low-rank updates.
+
+        TODO: similar to qNEHVI, when we are using sequential greedy candidate
+        selection, we could incorporate pending points X_baseline and compute
+        the incremental qNEI from the new point. This would greatly increase
+        efficiency for large batches.
         """
         super().__init__(
-            model=model, sampler=sampler, objective=objective, X_pending=X_pending
+            model=model,
+            sampler=sampler,
+            objective=objective,
+            posterior_transform=posterior_transform,
+            X_pending=X_pending,
         )
+        self._setup(model=model, sampler=self.sampler, cache_root=cache_root)
+        self.base_sampler = deepcopy(self.sampler)
         if prune_baseline:
             X_baseline = prune_inferior_points(
                 model=model,
                 X=X_baseline,
                 objective=objective,
+                posterior_transform=posterior_transform,
                 marginalize_dim=kwargs.get("marginalize_dim"),
             )
         self.register_buffer("X_baseline", X_baseline)
 
+        if self._cache_root:
+            self.q = -1
+            # set baseline samples
+            with torch.no_grad():
+                posterior = self.model.posterior(X_baseline)
+                baseline_samples = self.base_sampler(posterior)
+            baseline_obj = self.objective(baseline_samples, X=X_baseline)
+            self.register_buffer("baseline_samples", baseline_samples)
+            self.register_buffer(
+                "baseline_obj_max_values", baseline_obj.max(dim=-1).values
+            )
+            self._cache_root_decomposition(posterior=posterior)
+
+    def _set_sampler(
+        self,
+        q: int,
+        posterior: Posterior,
+    ) -> None:
+        r"""Update the sampler to use the original base samples for X_baseline.
+
+        Args:
+            q: the batch size
+            posterior: the posterior
+
+        TODO: refactor some/all of this into the MCSampler.
+        """
+        if self.q != q:
+            # create new base_samples
+            base_sample_shape = self.sampler._get_base_sample_shape(posterior=posterior)
+            self.sampler._construct_base_samples(
+                posterior=posterior, shape=base_sample_shape
+            )
+            if (
+                self.X_baseline.shape[0] > 0
+                and self.base_sampler.base_samples is not None
+            ):
+                current_base_samples = self.base_sampler.base_samples.detach().clone()
+                view_shape = (
+                    base_sample_shape[:1]
+                    + torch.Size([1] * (len(base_sample_shape) - 3))
+                    + current_base_samples.shape[-2:]
+                )
+                expanded_shape = (
+                    base_sample_shape[:-2] + current_base_samples.shape[-2:]
+                )
+                # Use stored base samples:
+                # Use all base_samples from the current sampler
+                # this includes the base_samples from the base_sampler
+                # and any base_samples for the new points in the sampler.
+                # For example, when using sequential greedy candidate generation
+                # then generate the new candidate point using last (-1) base_sample
+                # in sampler. This copies that base sample.
+                end_idx = current_base_samples.shape[-2]
+                self.sampler.base_samples[..., :end_idx, :] = current_base_samples.view(
+                    view_shape
+                ).expand(expanded_shape)
+            self.q = q
+
+    def _forward_cached(self, posterior: Posterior, X_full: Tensor, q: int) -> Tensor:
+        r"""Compute difference objective using cached root decomposition.
+
+        Args:
+            posterior: The posterior.
+            X_full: A `batch_shape x n + q x d`-dim tensor of inputs
+            q: The batch size.
+
+        Returns:
+            A `sample_shape x batch_shape`-dim tensor containing the
+                difference in objective under each MC sample.
+        """
+        self._set_sampler(q=q, posterior=posterior)
+        # handle one-to-many input transforms
+        n_w = posterior.event_shape[-2] // X_full.shape[-2]
+        new_samples = self._get_f_X_samples(posterior=posterior, q_in=n_w * q)
+        new_obj = self.objective(new_samples, X=X_full[..., -q:, :])
+        new_obj_max_values = new_obj.max(dim=-1).values
+        n_sample_dims = len(self.base_sampler.sample_shape)
+        view_shape = torch.Size(
+            [
+                *self.baseline_obj_max_values.shape[:n_sample_dims],
+                *(1,) * (new_obj_max_values.ndim - self.baseline_obj_max_values.ndim),
+                *self.baseline_obj_max_values.shape[n_sample_dims:],
+            ]
+        )
+        return new_obj_max_values - self.baseline_obj_max_values.view(view_shape)
+
     @concatenate_pending_points
     @t_batch_mode_transform()
     def forward(self, X: Tensor) -> Tensor:
@@ -238,13 +363,19 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
             of model and input `X`.
         """
         q = X.shape[-2]
-        X_full = torch.cat([X, match_batch_shape(self.X_baseline, X)], dim=-2)
+        X_full = torch.cat([match_batch_shape(self.X_baseline, X), X], dim=-2)
         # TODO: Implement more efficient way to compute posterior over both training and
         # test points in GPyTorch (https://github.com/cornellius-gp/gpytorch/issues/567)
-        posterior = self.model.posterior(X_full)
-        samples = self.sampler(posterior)
-        obj = self.objective(samples, X=X_full)
-        diffs = obj[..., :q].max(dim=-1).values - obj[..., q:].max(dim=-1).values
+        posterior = self.model.posterior(
+            X_full, posterior_transform=self.posterior_transform
+        )
+        if self._cache_root:
+            diffs = self._forward_cached(posterior=posterior, X_full=X_full, q=q)
+        else:
+            samples = self.sampler(posterior)
+            obj = self.objective(samples, X=X_full)
+            diffs = obj[..., -q:].max(dim=-1).values - obj[..., :-q].max(dim=-1).values
+
         return diffs.clamp_min(0).mean(dim=0)
 
 
@@ -273,6 +404,7 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
         best_f: Union[float, Tensor],
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCAcquisitionObjective] = None,
+        posterior_transform: Optional[PosteriorTransform] = None,
         X_pending: Optional[Tensor] = None,
         tau: float = 1e-3,
     ) -> None:
@@ -287,6 +419,7 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
                 `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`
             objective: The MCAcquisitionObjective under which the samples are
                 evaluated. Defaults to `IdentityMCObjective()`.
+            posterior_transform: A PosteriorTransform (optional).
             X_pending:  A `m x d`-dim Tensor of `m` design points that have
                 points that have been submitted for function evaluation
                 but have not yet been evaluated.  Concatenated into X upon
@@ -297,7 +430,11 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
                 estimates with higher variance.
         """
         super().__init__(
-            model=model, sampler=sampler, objective=objective, X_pending=X_pending
+            model=model,
+            sampler=sampler,
+            objective=objective,
+            posterior_transform=posterior_transform,
+            X_pending=X_pending,
         )
         self.register_buffer("best_f", torch.as_tensor(best_f, dtype=float))
         self.register_buffer("tau", torch.as_tensor(tau, dtype=float))
@@ -316,7 +453,9 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
             given design points `X`, where `batch_shape'` is the broadcasted batch shape
             of model and input `X`.
         """
-        posterior = self.model.posterior(X)
+        posterior = self.model.posterior(
+            X=X, posterior_transform=self.posterior_transform
+        )
         samples = self.sampler(posterior)
         obj = self.objective(samples, X=X)
         max_obj = obj.max(dim=-1)[0]
@@ -353,7 +492,9 @@ class qSimpleRegret(MCAcquisitionFunction):
             points `X`, where `batch_shape'` is the broadcasted batch shape of model
             and input `X`.
         """
-        posterior = self.model.posterior(X)
+        posterior = self.model.posterior(
+            X=X, posterior_transform=self.posterior_transform
+        )
         samples = self.sampler(posterior)
         obj = self.objective(samples, X=X)
         val = obj.max(dim=-1)[0].mean(dim=0)
@@ -382,6 +523,7 @@ class qUpperConfidenceBound(MCAcquisitionFunction):
         beta: float,
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCAcquisitionObjective] = None,
+        posterior_transform: Optional[PosteriorTransform] = None,
         X_pending: Optional[Tensor] = None,
     ) -> None:
         r"""q-Upper Confidence Bound.
@@ -393,13 +535,18 @@ class qUpperConfidenceBound(MCAcquisitionFunction):
                 `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`
             objective: The MCAcquisitionObjective under which the samples are
                 evaluated. Defaults to `IdentityMCObjective()`.
+            posterior_transform: A PosteriorTransform (optional).
             X_pending: A `batch_shape x m x d`-dim Tensor of `m` design points that have
                 points that have been submitted for function evaluation but have not yet
                 been evaluated. Concatenated into X upon forward call. Copied and set to
                 have no gradient.
         """
         super().__init__(
-            model=model, sampler=sampler, objective=objective, X_pending=X_pending
+            model=model,
+            sampler=sampler,
+            objective=objective,
+            posterior_transform=posterior_transform,
+            X_pending=X_pending,
         )
         self.beta_prime = math.sqrt(beta * math.pi / 2)
 
@@ -417,7 +564,9 @@ class qUpperConfidenceBound(MCAcquisitionFunction):
             design points `X`, where `batch_shape'` is the broadcasted batch shape of
             model and input `X`.
         """
-        posterior = self.model.posterior(X)
+        posterior = self.model.posterior(
+            X=X, posterior_transform=self.posterior_transform
+        )
         samples = self.sampler(posterior)
         obj = self.objective(samples, X=X)
         mean = obj.mean(dim=0)
