@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -27,10 +27,13 @@ import warnings
 from abc import abstractmethod
 from copy import deepcopy
 from itertools import combinations
-from typing import Union, Any, Callable, List, Optional
+from typing import Any, Callable, List, Optional, Union
 
 import torch
-from botorch.acquisition.acquisition import AcquisitionFunction
+from botorch.acquisition.acquisition import (
+    AcquisitionFunction,
+)
+from botorch.acquisition.cached_cholesky import CachedCholeskyMCAcquisitionFunction
 from botorch.acquisition.multi_objective.objective import (
     IdentityMCMultiOutputObjective,
     MCMultiOutputObjective,
@@ -41,15 +44,10 @@ from botorch.acquisition.multi_objective.utils import (
 from botorch.exceptions.errors import UnsupportedError
 from botorch.exceptions.warnings import BotorchWarning
 from botorch.models.model import Model
-from botorch.models.model_list_gp_regression import ModelListGP
-from botorch.models.multitask import MultiTaskGP
-from botorch.posteriors.gpytorch import GPyTorchPosterior
+from botorch.models.transforms.input import InputPerturbation
+from botorch.posteriors import DeterministicPosterior
 from botorch.posteriors.posterior import Posterior
 from botorch.sampling.samplers import MCSampler, SobolQMCNormalSampler
-from botorch.utils.low_rank import (
-    extract_batch_covar,
-    sample_cached_cholesky,
-)
 from botorch.utils.multi_objective.box_decompositions.box_decomposition_list import (
     BoxDecompositionList,
 )
@@ -70,8 +68,6 @@ from botorch.utils.transforms import (
     match_batch_shape,
     t_batch_mode_transform,
 )
-from gpytorch import settings as gpt_settings
-from gpytorch.utils.errors import NotPSDError, NanError
 from torch import Tensor
 
 
@@ -83,6 +79,7 @@ class MultiObjectiveMCAcquisitionFunction(AcquisitionFunction):
         model: Model,
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCMultiOutputObjective] = None,
+        constraints: Optional[List[Callable[[Tensor], Tensor]]] = None,
         X_pending: Optional[Tensor] = None,
     ) -> None:
         r"""Constructor for the MCAcquisitionFunction base class.
@@ -93,6 +90,10 @@ class MultiObjectiveMCAcquisitionFunction(AcquisitionFunction):
                 `SobolQMCNormalSampler(num_samples=128, collapse_batch_dims=True)`.
             objective: The MCMultiOutputObjective under which the samples are
                 evaluated. Defaults to `IdentityMultiOutputObjective()`.
+            constraints: A list of callables, each mapping a Tensor of dimension
+                `sample_shape x batch-shape x q x m` to a Tensor of dimension
+                `sample_shape x batch-shape x q`, where negative values imply
+                feasibility.
             X_pending:  A `m x d`-dim Tensor of `m` design points that have
                 points that have been submitted for function evaluation
                 but have not yet been evaluated.
@@ -108,7 +109,19 @@ class MultiObjectiveMCAcquisitionFunction(AcquisitionFunction):
                 "Only objectives of type MCMultiOutputObjective are supported for "
                 "Multi-Objective MC acquisition functions."
             )
+        if (
+            hasattr(model, "input_transform")
+            and isinstance(model.input_transform, InputPerturbation)
+            and constraints is not None
+        ):
+            raise UnsupportedError(
+                "Constraints are not supported with input perturbations, due to"
+                "sample q-batch shape being different than that of the inputs."
+                "Use a composite objective that applies feasibility weighting to"
+                "samples before calculating the risk measure."
+            )
         self.add_module("objective", objective)
+        self.constraints = constraints
         self.X_pending = None
         if X_pending is not None:
             self.set_X_pending(X_pending)
@@ -184,58 +197,66 @@ class qExpectedHypervolumeImprovement(MultiObjectiveMCAcquisitionFunction):
             device=partitioning.pareto_Y.device,
         )
         super().__init__(
-            model=model, sampler=sampler, objective=objective, X_pending=X_pending
+            model=model,
+            sampler=sampler,
+            objective=objective,
+            constraints=constraints,
+            X_pending=X_pending,
         )
-        self.constraints = constraints
         self.eta = eta
         self.register_buffer("ref_point", ref_point)
         cell_bounds = partitioning.get_hypercell_bounds()
         self.register_buffer("cell_lower_bounds", cell_bounds[0])
         self.register_buffer("cell_upper_bounds", cell_bounds[1])
-        self.q = -1
+        self.q_out = -1
         self.q_subset_indices = BufferDict()
 
-    def _cache_q_subset_indices(self, q: int) -> None:
-        r"""Cache indices corresponding to all subsets of `q`.
+    def _cache_q_subset_indices(self, q_out: int) -> None:
+        r"""Cache indices corresponding to all subsets of `q_out`.
 
         This means that consecutive calls to `forward` with the same
-        `q` will not recompute the indices for all (2^q - 1) subsets.
+        `q_out` will not recompute the indices for all (2^q_out - 1) subsets.
 
         Note: this will use more memory than regenerating the indices
         for each i and then deleting them, but it will be faster for
         repeated evaluations (e.g. during optimization).
 
         Args:
-            q: batch size
+            q_out: The batch size of the objectives. This is typically equal
+                to the q-batch size of `X`. However, if using a set valued
+                objective (e.g., MVaR) that produces `s` objective values for
+                each point on the q-batch of `X`, we need to properly account
+                for each objective while calculating the hypervolume contributions
+                by using `q_out = q * s`.
         """
-        if q != self.q:
-            indices = list(range(q))
-            tkwargs = {"dtype": torch.long, "device": self.cell_lower_bounds.device}
+        if q_out != self.q_out:
+            indices = list(range(q_out))
+            tkwargs = {"dtype": torch.long, "device": self.ref_point.device}
             self.q_subset_indices = BufferDict(
                 {
                     f"q_choose_{i}": torch.tensor(
                         list(combinations(indices, i)), **tkwargs
                     )
-                    for i in range(1, q + 1)
+                    for i in range(1, q_out + 1)
                 }
             )
-            self.q = q
+            self.q_out = q_out
 
     def _compute_qehvi(self, samples: Tensor, X: Optional[Tensor] = None) -> Tensor:
         r"""Compute the expected (feasible) hypervolume improvement given MC samples.
 
         Args:
-            samples: A `n_samples x batch_shape x q x m`-dim tensor of samples.
+            samples: A `n_samples x batch_shape x q' x m`-dim tensor of samples.
             X: A `batch_shape x q x d`-dim tensor of inputs.
 
         Returns:
             A `batch_shape x (model_batch_shape)`-dim tensor of expected hypervolume
             improvement for each batch.
         """
-        q = samples.shape[-2]
         # Note that the objective may subset the outcomes (e.g. this will usually happen
         # if there are constraints present).
         obj = self.objective(samples, X=X)
+        q = obj.shape[-2]
         if self.constraints is not None:
             feas_weights = torch.ones(
                 obj.shape[:-1], device=obj.device, dtype=obj.dtype
@@ -246,8 +267,8 @@ class qExpectedHypervolumeImprovement(MultiObjectiveMCAcquisitionFunction):
                 samples=samples,
                 eta=self.eta,
             )
-        self._cache_q_subset_indices(q=q)
-        batch_shape = samples.shape[:-2]
+        self._cache_q_subset_indices(q_out=q)
+        batch_shape = obj.shape[:-2]
         # this is n_samples x input_batch_shape x
         areas_per_segment = torch.zeros(
             *batch_shape,
@@ -269,9 +290,9 @@ class qExpectedHypervolumeImprovement(MultiObjectiveMCAcquisitionFunction):
             1,
             self.cell_upper_bounds.shape[-1],
         )
-        for i in range(1, q + 1):
+        for i in range(1, self.q_out + 1):
             # TODO: we could use batches to compute (q choose i) and (q choose q-i)
-            # simulataneously since subsets of size i and q-i have the same number of
+            # simultaneously since subsets of size i and q-i have the same number of
             # elements. This would decrease the number of iterations, but increase
             # memory usage.
             q_choose_i = self.q_subset_indices[f"q_choose_{i}"]
@@ -315,10 +336,12 @@ class qExpectedHypervolumeImprovement(MultiObjectiveMCAcquisitionFunction):
     def forward(self, X: Tensor) -> Tensor:
         posterior = self.model.posterior(X)
         samples = self.sampler(posterior)
-        return self._compute_qehvi(samples=samples)
+        return self._compute_qehvi(samples=samples, X=X)
 
 
-class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
+class qNoisyExpectedHypervolumeImprovement(
+    qExpectedHypervolumeImprovement, CachedCholeskyMCAcquisitionFunction
+):
     def __init__(
         self,
         model: Model,
@@ -398,23 +421,15 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
             ref_point, dtype=X_baseline.dtype, device=X_baseline.device
         )
         super(qExpectedHypervolumeImprovement, self).__init__(
-            model=model, sampler=sampler, objective=objective
+            model=model,
+            sampler=sampler,
+            objective=objective,
+            constraints=constraints,
         )
-        if not self.sampler.collapse_batch_dims:
-            raise UnsupportedError(
-                "qNoisyExpectedHypervolumeImprovement currently requires sampler "
-                "to use `collapse_batch_dims=True`."
-            )
-        elif self.sampler.base_samples is not None:
-            warnings.warn(
-                message=(
-                    "sampler.base_samples is not None. qNEHVI requires that the "
-                    "base_samples be initialized to None. Resetting "
-                    "sampler.base_samples to None."
-                ),
-                category=BotorchWarning,
-            )
-            self.sampler.base_samples = None
+        self._setup(
+            model=model, sampler=self.sampler, cache_root=cache_root, check_sampler=True
+        )
+
         if X_baseline.ndim > 2:
             raise UnsupportedError(
                 "qNoisyExpectedHypervolumeImprovement does not support batched "
@@ -432,12 +447,10 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
         self.register_buffer("ref_point", ref_point)
         self.base_sampler = deepcopy(self.sampler)
 
-        self.constraints = constraints
         self.alpha = alpha
         self.eta = eta
-        models = model.models if isinstance(model, ModelListGP) else [model]
-        self._is_mt = any(isinstance(m, MultiTaskGP) for m in models)
-        self.q = -1
+        self.q_in = -1
+        self.q_out = -1
         self.q_subset_indices = BufferDict()
         self.partitioning = None
         # set partitioning class and args
@@ -449,7 +462,6 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
             self.p_class = FastNondominatedPartitioning
         self.register_buffer("_X_baseline", X_baseline)
         self.register_buffer("_X_baseline_and_pending", X_baseline)
-        self._cache_root = cache_root
         self.register_buffer(
             "cache_pending",
             torch.tensor(cache_pending, dtype=bool),
@@ -468,11 +480,11 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
         )
 
         if X_pending is not None:
-            # this will call self._set_cell_bounds if the number of pending
-            # points is greater than self._max_iep
+            # This will call self._set_cell_bounds if the number of pending
+            # points is greater than self._max_iep.
             self.set_X_pending(X_pending)
         # In the case that X_pending is not None, but there are fewer than
-        # max_iep pending points, the box decompoisitions are not performed in
+        # max_iep pending points, the box decompositions are not performed in
         # set_X_pending. Therefore, we need to perform a box decomposition over
         # f(X_baseline) here.
         if X_pending is None or X_pending.shape[-2] <= self._max_iep:
@@ -490,7 +502,7 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
             obj: A `sample_shape x batch_shape x n x m`-dim tensor of samples
                 of objectives.
             feas: `sample_shape x batch_shape x n`-dim tensor of samples
-                of feasiblity indicators.
+                of feasibility indicators.
         """
         initial_hvs = []
         for i, sample in enumerate(obj):
@@ -509,21 +521,6 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
             ),
         )
 
-    def _cache_root_decomposition(
-        self,
-        posterior: GPyTorchPosterior,
-    ) -> None:
-        r"""Cache the root decomposition of the covariance of `f(X_baseline)`.
-
-        Args:
-            posterior: The posterior over f(X_baseline).
-        """
-        lazy_covar = extract_batch_covar(posterior.mvn)
-        with gpt_settings.fast_computations.covar_root_decomposition(False):
-            lazy_covar_root = lazy_covar.root_decomposition()
-            baseline_L = lazy_covar_root.root.evaluate()
-        self.register_buffer("_baseline_L", baseline_L)
-
     def _set_cell_bounds(self, num_new_points: int) -> None:
         r"""Compute the box decomposition under each posterior sample.
 
@@ -537,9 +534,10 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
         if self.X_baseline.shape[0] > 0:
             with torch.no_grad():
                 posterior = self.model.posterior(self.X_baseline)
-            # reset sampler
-            self.q = -1
-            self._set_sampler(q=num_new_points, posterior=posterior)
+            # Reset sampler, accounting for possible one-to-many transform.
+            self.q_in = -1
+            n_w = posterior.event_shape[-2] // self.X_baseline.shape[-2]
+            self._set_sampler(q_in=num_new_points * n_w, posterior=posterior)
             # set base_sampler
             self.base_sampler.register_buffer(
                 "base_samples", self.sampler.base_samples.detach().clone()
@@ -549,7 +547,7 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
             # cache posterior
             if self._cache_root:
                 self._cache_root_decomposition(posterior=posterior)
-            obj = self.objective(samples)
+            obj = self.objective(samples, X=self.X_baseline)
             if self.constraints is not None:
                 feas = torch.stack(
                     [c(samples) <= 0 for c in self.constraints], dim=0
@@ -625,15 +623,15 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
             X_pending = X_pending.detach().clone()
             if self.cache_pending:
                 X_baseline = torch.cat([self._X_baseline, X_pending], dim=-2)
-                # number of new points is the total number of points minus
+                # Number of new points is the total number of points minus
                 # (the number of previously cached pending points plus the
-                # of number of baseline points)
+                # of number of baseline points).
                 num_new_points = X_baseline.shape[0] - self.X_baseline.shape[0]
                 if num_new_points > 0:
                     if num_new_points > self._max_iep:
-                        # set the new baseline points to include pending points
+                        # Set the new baseline points to include pending points.
                         self.register_buffer("_X_baseline_and_pending", X_baseline)
-                        # recompute box decompositions
+                        # Recompute box decompositions.
                         self._set_cell_bounds(num_new_points=num_new_points)
                         if not self.incremental_nehvi:
                             self._prev_nehvi = (
@@ -641,12 +639,12 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
                                 .clamp_min(0.0)
                                 .mean()
                             )
-                        # set to None so that pending points are not concatenated in
-                        # forward
+                        # Set to None so that pending points are not concatenated in
+                        # forward.
                         self.X_pending = None
-                        # set q=-1 to so that self.sampler is updated at the next
-                        # forward call
-                        self.q = -1
+                        # Set q_in=-1 to so that self.sampler is updated at the next
+                        # forward call.
+                        self.q_in = -1
                     else:
                         self.X_pending = X_pending[-num_new_points:]
             else:
@@ -667,18 +665,22 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
 
     def _set_sampler(
         self,
-        q: int,
+        q_in: int,
         posterior: Posterior,
     ) -> None:
         r"""Update the sampler to use the original base samples for X_baseline.
 
         Args:
-            q: the batch size
-            posterior: the posterior
+            q_in: The effective input batch size. This is typically equal to the
+                q-batch size of `X`. However, if using a one-to-many input transform,
+                e.g., `InputPerturbation` with `n_w` perturbations, the posterior will
+                have `n_w` points on the q-batch for each point on the q-batch of `X`.
+                In which case, `q_in = q * n_w` is used.
+            posterior: The posterior.
 
         TODO: refactor some/all of this into the MCSampler.
         """
-        if self.q != q:
+        if self.q_in != q_in:
             # create new base_samples
             base_sample_shape = self.sampler._get_base_sample_shape(posterior=posterior)
             self.sampler._construct_base_samples(
@@ -687,15 +689,22 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
             if (
                 self.X_baseline.shape[0] > 0
                 and self.base_sampler.base_samples is not None
+                and not isinstance(posterior, DeterministicPosterior)
             ):
                 current_base_samples = self.base_sampler.base_samples.detach().clone()
+                # This is the # of non-`sample_shape` dimensions.
+                base_ndims = current_base_samples.dim() - 1
+                # Unsqueeze as many dimensions as needed to match base_sample_shape.
                 view_shape = (
-                    base_sample_shape[0:1]
-                    + torch.Size([1] * (len(base_sample_shape) - 3))
-                    + current_base_samples.shape[-2:]
+                    self.sampler.sample_shape
+                    + torch.Size(
+                        [1] * (len(base_sample_shape) - current_base_samples.dim())
+                    )
+                    + current_base_samples.shape[-base_ndims:]
                 )
                 expanded_shape = (
-                    base_sample_shape[:-2] + current_base_samples.shape[-2:]
+                    base_sample_shape[:-base_ndims]
+                    + current_base_samples.shape[-base_ndims:]
                 )
                 # Use stored base samples:
                 # Use all base_samples from the current sampler
@@ -704,50 +713,23 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
                 # For example, when using sequential greedy candidate generation
                 # then generate the new candidate point using last (-1) base_sample
                 # in sampler. This copies that base sample.
-                end_idx = current_base_samples.shape[-2]
-                self.sampler.base_samples[..., :end_idx, :] = current_base_samples.view(
-                    view_shape
-                ).expand(expanded_shape)
-                # update cached subset indices
-                # Note: this also stores self.q = q
-                self._cache_q_subset_indices(q=q)
-
-    def _get_f_X_samples(self, posterior: GPyTorchPosterior, q: int) -> Tensor:
-        r"""Get posterior samples at the `q` new points from the joint posterior.
-
-        Args:
-            posterior: The joint posterior is over (X_baseline, X).
-            q: The number of new points in X.
-
-        Returns:
-            A `sample_shape x batch_shape x q x m`-dim tensor of posterior
-                samples at the new points.
-        """
-        # technically we should make sure that we add a consistent nugget to the
-        # cached covariance (and box decompositions)
-        # and the new block.
-        # But recomputing box decompositions every time the jitter changes would
-        # be quite slow
-        if not self._is_mt and self._cache_root and hasattr(self, "_baseline_L"):
-            try:
-                return sample_cached_cholesky(
-                    posterior=posterior,
-                    baseline_L=self._baseline_L,
-                    q=q,
-                    base_samples=self.sampler.base_samples,
-                    sample_shape=self.sampler.sample_shape,
+                expanded_samples = current_base_samples.view(view_shape).expand(
+                    expanded_shape
                 )
-            except (NanError, NotPSDError):
-                warnings.warn(
-                    "Low-rank cholesky updates failed due NaNs or due to an "
-                    "ill-conditioned covariance matrix. "
-                    "Falling back to standard sampling.",
-                    BotorchWarning,
-                )
-
-        # TODO: improve efficiency for multi-task models
-        samples = self.sampler(posterior)
-        return samples[..., -q:, :]
+                if self._uses_matheron:
+                    n_train_samples = current_base_samples.shape[-1] // 2
+                    # The train base samples.
+                    self.sampler.base_samples[..., :n_train_samples] = expanded_samples[
+                        ..., :n_train_samples
+                    ]
+                    # The train noise base samples.
+                    self.sampler.base_samples[
+                        ..., -n_train_samples:
+                    ] = expanded_samples[..., -n_train_samples:]
+                else:
+                    end_idx = current_base_samples.shape[-2]
+                    self.sampler.base_samples[..., :end_idx, :] = expanded_samples
+                self.q_in = q_in
 
     @concatenate_pending_points
     @t_batch_mode_transform()
@@ -756,12 +738,14 @@ class qNoisyExpectedHypervolumeImprovement(qExpectedHypervolumeImprovement):
         # Note: it is important to compute the full posterior over `(X_baseline, X)`
         # to ensure that we properly sample `f(X)` from the joint distribution `
         # `f(X_baseline, X) ~ P(f | D)` given that we can already fixed the sampled
-        # function values for `f(X_baseline)`
+        # function values for `f(X_baseline)`.
         # TODO: improve efficiency by not recomputing baseline-baseline
-        # covariance matrix
+        # covariance matrix.
         posterior = self.model.posterior(X_full)
-        q = X.shape[-2]
-        self._set_sampler(q=q, posterior=posterior)
-        samples = self._get_f_X_samples(posterior=posterior, q=q)
-        # add previous nehvi from pending points
-        return self._compute_qehvi(samples=samples) + self._prev_nehvi
+        # Account for possible one-to-many transform.
+        n_w = posterior.event_shape[X_full.dim() - 2] // X_full.shape[-2]
+        q_in = X.shape[-2] * n_w
+        self._set_sampler(q_in=q_in, posterior=posterior)
+        samples = self._get_f_X_samples(posterior=posterior, q_in=q_in)
+        # Add previous nehvi from pending points.
+        return self._compute_qehvi(samples=samples, X=X) + self._prev_nehvi
