@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -20,6 +20,7 @@ from copy import deepcopy
 from typing import Any, Iterator, List, Optional, Tuple, Union
 
 import torch
+from botorch.acquisition.objective import PosteriorTransform
 from botorch.exceptions.errors import BotorchTensorDimensionError
 from botorch.exceptions.warnings import BotorchTensorDimensionWarning
 from botorch.models.model import Model, ModelList
@@ -115,7 +116,11 @@ class GPyTorchModel(Model, ABC):
         return self._num_outputs
 
     def posterior(
-        self, X: Tensor, observation_noise: Union[bool, Tensor] = False, **kwargs: Any
+        self,
+        X: Tensor,
+        observation_noise: Union[bool, Tensor] = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
+        **kwargs: Any,
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
 
@@ -126,6 +131,7 @@ class GPyTorchModel(Model, ABC):
             observation_noise: If True, add the observation noise from the
                 likelihood to the posterior. If a Tensor, use it directly as the
                 observation noise (must be of shape `(batch_shape) x q`).
+            posterior_transform: An optional PosteriorTransform.
 
         Returns:
             A `GPyTorchPosterior` object, representing a batch of `b` joint
@@ -150,6 +156,8 @@ class GPyTorchModel(Model, ABC):
         posterior = GPyTorchPosterior(mvn=mvn)
         if hasattr(self, "outcome_transform"):
             posterior = self.outcome_transform.untransform_posterior(posterior)
+        if posterior_transform is not None:
+            return posterior_transform(posterior)
         return posterior
 
     def condition_on_observations(self, X: Tensor, Y: Tensor, **kwargs: Any) -> Model:
@@ -296,6 +304,7 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
         X: Tensor,
         output_indices: Optional[List[int]] = None,
         observation_noise: Union[bool, Tensor] = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
@@ -312,6 +321,7 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
             observation_noise: If True, add the observation noise from the
                 likelihood to the posterior. If a Tensor, use it directly as the
                 observation noise (must be of shape `(batch_shape) x q x m`).
+            posterior_transform: An optional PosteriorTransform.
 
         Returns:
             A `GPyTorchPosterior` object, representing `batch_shape` joint
@@ -357,6 +367,8 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
         posterior = GPyTorchPosterior(mvn=mvn)
         if hasattr(self, "outcome_transform"):
             posterior = self.outcome_transform.untransform_posterior(posterior)
+        if posterior_transform is not None:
+            return posterior_transform(posterior)
         return posterior
 
     def condition_on_observations(
@@ -508,6 +520,7 @@ class ModelListGPyTorchModel(GPyTorchModel, ModelList, ABC):
         X: Tensor,
         output_indices: Optional[List[int]] = None,
         observation_noise: Union[bool, Tensor] = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
@@ -526,6 +539,7 @@ class ModelListGPyTorchModel(GPyTorchModel, ModelList, ABC):
                 `(batch_shape) x q x m`, use it directly as the observation
                 noise (with `observation_noise[...,i]` added to the posterior
                 of the `i`-th model).
+            posterior_transform: An optional PosteriorTransform.
 
         Returns:
             A `GPyTorchPosterior` object, representing `batch_shape` joint
@@ -582,10 +596,14 @@ class ModelListGPyTorchModel(GPyTorchModel, ModelList, ABC):
             mvns.append(tf_mvn)
         # return result as a GPyTorchPosteriors
         if len(mvns) == 1:
-            return GPyTorchPosterior(mvn=mvns[0])
-        return GPyTorchPosterior(
-            mvn=MultitaskMultivariateNormal.from_independent_mvns(mvns=mvns)
-        )
+            posterior = GPyTorchPosterior(mvn=mvns[0])
+        else:
+            posterior = GPyTorchPosterior(
+                mvn=MultitaskMultivariateNormal.from_independent_mvns(mvns=mvns)
+            )
+        if posterior_transform is not None:
+            return posterior_transform(posterior)
+        return posterior
 
     def condition_on_observations(self, X: Tensor, Y: Tensor, **kwargs: Any) -> Model:
         raise NotImplementedError()
@@ -603,6 +621,7 @@ class MultiTaskGPyTorchModel(GPyTorchModel, ABC):
         X: Tensor,
         output_indices: Optional[List[int]] = None,
         observation_noise: Union[bool, Tensor] = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
@@ -619,6 +638,7 @@ class MultiTaskGPyTorchModel(GPyTorchModel, ABC):
             observation_noise: If True, add observation noise from the respective
                 likelihoods. If a Tensor, specifies the observation noise levels
                 to add.
+            posterior_transform: An optional PosteriorTransform.
 
         Returns:
             A `GPyTorchPosterior` object, representing `batch_shape` joint
@@ -648,14 +668,17 @@ class MultiTaskGPyTorchModel(GPyTorchModel, ABC):
                 )
         # If single-output, return the posterior of a single-output model
         if num_outputs == 1:
-            return GPyTorchPosterior(mvn=mvn)
-        # Otherwise, make a MultitaskMultivariateNormal out of this
-        mtmvn = MultitaskMultivariateNormal(
-            mean=mvn.mean.view(*X.shape[:-2], num_outputs, -1).transpose(-1, -2),
-            covariance_matrix=mvn.lazy_covariance_matrix,
-            interleaved=False,
-        )
-        posterior = GPyTorchPosterior(mvn=mtmvn)
+            posterior = GPyTorchPosterior(mvn=mvn)
+        else:
+            # Otherwise, make a MultitaskMultivariateNormal out of this
+            mtmvn = MultitaskMultivariateNormal(
+                mean=mvn.mean.view(*X.shape[:-2], num_outputs, -1).transpose(-1, -2),
+                covariance_matrix=mvn.lazy_covariance_matrix,
+                interleaved=False,
+            )
+            posterior = GPyTorchPosterior(mvn=mtmvn)
         if hasattr(self, "outcome_transform"):
             posterior = self.outcome_transform.untransform_posterior(posterior)
+        if posterior_transform is not None:
+            return posterior_transform(posterior)
         return posterior
