@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -25,9 +25,9 @@ import math
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 import torch
+from botorch.acquisition.objective import PosteriorTransform
 from botorch.models.gp_regression import MIN_INFERRED_NOISE_LEVEL
-from botorch.models.gpytorch import GPyTorchModel
-from botorch.models.gpytorch import MultiTaskGPyTorchModel
+from botorch.models.gpytorch import GPyTorchModel, MultiTaskGPyTorchModel
 from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import OutcomeTransform
 from botorch.posteriors.multitask import MultitaskGPPosterior
@@ -43,9 +43,10 @@ from gpytorch.kernels.multitask_kernel import MultitaskKernel
 from gpytorch.kernels.scale_kernel import ScaleKernel
 from gpytorch.lazy import (
     BatchRepeatLazyTensor,
+    CatLazyTensor,
     DiagLazyTensor,
-    KroneckerProductLazyTensor,
     KroneckerProductDiagLazyTensor,
+    KroneckerProductLazyTensor,
     lazify,
     RootLazyTensor,
 )
@@ -101,7 +102,8 @@ class MultiTaskGP(ExactGP, MultiTaskGPyTorchModel):
             train_X: A `n x (d + 1)` or `b x n x (d + 1)` (batch mode) tensor
                 of training data. One of the columns should contain the task
                 features (see `task_feature` argument).
-            train_Y: A `n` or `b x n` (batch mode) tensor of training observations.
+            train_Y: A `n x 1` or `b x n x 1` (batch mode) tensor of training
+                observations.
             task_feature: The index of the task feature (`-d <= task_feature <= d`).
             output_tasks: A list of task indices for which to compute model
                 outputs for. If omitted, return outputs for all task indices.
@@ -316,7 +318,7 @@ class FixedNoiseMultiTaskGP(MultiTaskGP):
             train_X: A `n x (d + 1)` or `b x n x (d + 1)` (batch mode) tensor
                 of training data. One of the columns should contain the task
                 features (see `task_feature` argument).
-            train_Y: A `n` or `b x n` (batch mode) tensor of training
+            train_Y: A `n x 1` or `b x n x 1` (batch mode) tensor of training
                 observations.
             train_Yvar: A `n` or `b x n` (batch mode) tensor of observation
                 noise standard errors.
@@ -403,6 +405,7 @@ class KroneckerMultiTaskGP(ExactGP, GPyTorchModel):
         train_X: Tensor,
         train_Y: Tensor,
         likelihood: Optional[MultitaskGaussianLikelihood] = None,
+        data_covar_module: Optional[Module] = None,
         task_covar_prior: Optional[Prior] = None,
         rank: Optional[int] = None,
         input_transform: Optional[InputTransform] = None,
@@ -417,6 +420,8 @@ class KroneckerMultiTaskGP(ExactGP, GPyTorchModel):
             likelihood: A `MultitaskGaussianLikelihood`. If omitted, uses a
                 `MultitaskGaussianLikelihood` with a `GammaPrior(1.1, 0.05)`
                 noise prior.
+            data_covar_module: The module computing the covariance (Kernel) matrix
+                in data space. If omitted, use a `MaternKernel`.
             task_covar_prior : A Prior on the task covariance matrix. Must operate
                 on p.s.d. matrices. A common prior for this is the `LKJ` prior. If
                 omitted, uses `LKJCovariancePrior` with `eta` parameter as specified
@@ -478,13 +483,18 @@ class KroneckerMultiTaskGP(ExactGP, GPyTorchModel):
         self.mean_module = MultitaskMean(
             base_means=ConstantMean(batch_shape=batch_shape), num_tasks=num_tasks
         )
-        self.covar_module = MultitaskKernel(
-            data_covar_module=MaternKernel(
+        if data_covar_module is None:
+            data_covar_module = MaternKernel(
                 nu=2.5,
                 ard_num_dims=ard_num_dims,
                 lengthscale_prior=GammaPrior(3.0, 6.0),
                 batch_shape=batch_shape,
-            ),
+            )
+        else:
+            data_covar_module = data_covar_module
+
+        self.covar_module = MultitaskKernel(
+            data_covar_module=data_covar_module,
             num_tasks=num_tasks,
             rank=rank,
             batch_shape=batch_shape,
@@ -545,10 +555,18 @@ class KroneckerMultiTaskGP(ExactGP, GPyTorchModel):
         X: Tensor,
         output_indices: Optional[List[int]] = None,
         observation_noise: Union[bool, Tensor] = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
         **kwargs: Any,
     ) -> MultitaskGPPosterior:
         self.eval()
 
+        if posterior_transform is not None:
+            # this could be very costly, disallow for now
+            raise NotImplementedError(
+                "Posterior transforms currently not supported for "
+                f"{self.__class__.__name__}"
+            )
+
         X = self.transform_inputs(X)
         train_x = self.transform_inputs(self.train_inputs[0])
 
@@ -572,6 +590,29 @@ class KroneckerMultiTaskGP(ExactGP, GPyTorchModel):
         # decomposition
         data_data_evals, data_data_evecs = data_data_covar.diagonalization()
 
+        # pad the eigenvalue and eigenvectors with zeros if we are using lanczos
+        if data_data_evecs.shape[-1] < data_data_evecs.shape[-2]:
+            cols_to_add = data_data_evecs.shape[-2] - data_data_evecs.shape[-1]
+            zero_evecs = torch.zeros(
+                *data_data_evecs.shape[:-1],
+                cols_to_add,
+                dtype=data_data_evals.dtype,
+                device=data_data_evals.device,
+            )
+            zero_evals = torch.zeros(
+                *data_data_evecs.shape[:-2],
+                cols_to_add,
+                dtype=data_data_evals.dtype,
+                device=data_data_evals.device,
+            )
+            data_data_evecs = CatLazyTensor(
+                data_data_evecs,
+                lazify(zero_evecs),
+                dim=-1,
+                output_device=data_data_evals.device,
+            )
+            data_data_evals = torch.cat((data_data_evals, zero_evals), dim=-1)
+
         # construct K_{xt, x}
         test_data_covar = self.covar_module.data_covar_module(X, train_x)
         # construct K_{xt, xt}
@@ -711,7 +752,6 @@ class KroneckerMultiTaskGP(ExactGP, GPyTorchModel):
 
         if hasattr(self, "outcome_transform"):
             posterior = self.outcome_transform.untransform_posterior(posterior)
-
         return posterior
 
     def train(self, val=True, *args, **kwargs):
