@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -27,6 +27,7 @@ from typing import Any, List, Optional, Tuple, Union
 
 import numpy as np
 import torch
+from botorch.acquisition.objective import PosteriorTransform
 from botorch.models.model import Model
 from botorch.models.transforms.input import InputTransform
 from botorch.posteriors.gpytorch import GPyTorchPosterior
@@ -45,7 +46,7 @@ from gpytorch.priors.smoothed_box_prior import SmoothedBoxPrior
 from gpytorch.priors.torch_priors import GammaPrior
 from gpytorch.utils.cholesky import psd_safe_cholesky
 from scipy import optimize
-from torch import Tensor, float32, float64
+from torch import float32, float64, Tensor
 
 
 class PairwiseGP(Model, GP):
@@ -914,6 +915,7 @@ class PairwiseGP(Model, GP):
         X: Tensor,
         output_indices: Optional[List[int]] = None,
         observation_noise: bool = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
         **kwargs: Any,
     ) -> Posterior:
         r"""Computes the posterior over model outputs at the provided points.
@@ -924,6 +926,7 @@ class PairwiseGP(Model, GP):
             output_indices: As defined in parent Model class, not used for this model.
             observation_noise: Ignored (since noise is not identifiable from scale
                 in probit models).
+            posterior_transform: An optional PosteriorTransform.
 
         Returns:
             A `Posterior` object, representing joint
@@ -938,8 +941,11 @@ class PairwiseGP(Model, GP):
             )
 
         post = self(X)
-
-        return GPyTorchPosterior(post)
+        posterior = GPyTorchPosterior(post)
+        if posterior_transform is not None:
+            return posterior_transform(posterior)
+        else:
+            return posterior
 
     def condition_on_observations(self, X: Tensor, Y: Tensor, **kwargs: Any) -> Model:
         r"""Condition the model on new observations.
