@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -28,8 +28,8 @@ from botorch.acquisition.knowledge_gradient import (
 from botorch.acquisition.utils import is_nonnegative
 from botorch.exceptions.errors import BotorchTensorDimensionError
 from botorch.exceptions.warnings import (
-    BotorchWarning,
     BadInitialCandidatesWarning,
+    BotorchWarning,
     SamplingWarning,
 )
 from botorch.models.model import Model
@@ -38,10 +38,10 @@ from botorch.utils.multi_objective.pareto import is_non_dominated
 from botorch.utils.sampling import (
     batched_multinomial,
     draw_sobol_samples,
-    manual_seed,
     get_polytope_samples,
+    manual_seed,
 )
-from botorch.utils.transforms import unnormalize, normalize, standardize
+from botorch.utils.transforms import normalize, standardize, unnormalize
 from torch import Tensor
 from torch.distributions import Normal
 from torch.quasirandom import SobolEngine
@@ -394,7 +394,8 @@ def gen_value_function_initial_conditions(
     # compute maximizer of the current value function
     value_function = _get_value_function(
         model=current_model,
-        objective=acq_function.objective,
+        objective=getattr(acq_function, "objective", None),
+        posterior_transform=acq_function.posterior_transform,
         sampler=getattr(acq_function, "sampler", None),
         project=getattr(acq_function, "project", None),
     )
