@@ -1,5 +1,5 @@
 #! /usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,17 +9,27 @@ Multi-objective optimization benchmark problems.
 
 References
 
+.. [Daulton2022]
+    S. Daulton, S. Cakmak, M. Balandat, M. A. Osborne, E. Zhou, and E. Bakshy.
+    Robust Multi-Objective Bayesian Optimization Under Input Noise. 2022.
+
 .. [Deb2005dtlz]
-    K. Deb, L. Thiele, M. Laumanns, E. Zitzler, A. Abraham, L. Jain, R. Goldberg.
-    "Scalable test problems for evolutionary multi-objective optimization"
-    in Evolutionary Multiobjective Optimization, London, U.K.: Springer-Verlag,
+    K. Deb, L. Thiele, M. Laumanns, E. Zitzler, A. Abraham, L. Jain, and
+    R. Goldberg. Scalable test problems for evolutionary multi-objective
+    optimization. Evolutionary Multiobjective Optimization, Springer-Verlag,
     pp. 105-145, 2005.
 
 .. [Deb2005robust]
-    K. Deb, H. Gupta. "Searching for Robust Pareto-Optimal Solutions in
-    Multi-objective Optimization" in Evolutionary Multi-Criterion Optimization,
+    K. Deb and H. Gupta. Searching for Robust Pareto-Optimal Solutions in
+    Multi-objective Optimization. Evolutionary Multi-Criterion Optimization,
     Springer-Berlin, pp. 150-164, 2005.
 
+.. [Frohlich2020]
+    L. Frohlich, E. Klenske, J. Vinogradska, C. Daniel, and M. Zeilinger.
+    Noisy-Input Entropy Search for Efficient Robust Bayesian Optimization.
+    Proceedings of the Twenty Third International Conference on Artificial
+    Intelligence and Statistics, PMLR 108:2262-2272, 2020.
+
 .. [GarridoMerchan2020]
     E. C. Garrido-Merch ́an and D. Hern ́andez-Lobato. Parallel Predictive Entropy
     Search for Multi-objective Bayesian Optimization with Constraints.
@@ -31,43 +41,54 @@ References
     Conference on Uncertainty in Artificial Intelligence (UAI’14).
     AUAI Press, Arlington, Virginia, USA, 250–259.
 
+.. [Liang2021]
+    Q. Liang and L. Lai, Scalable Bayesian Optimization Accelerates Process
+    Optimization of Penicillin Production. NeurIPS 2021 AI for Science Workshop, 2021.
+
+.. [Ma2019]
+    Z. Ma and Y. Wang. Evolutionary Constrained Multiobjective Optimization:
+    Test Suite Construction and Performance Comparisons. IEEE Transactions
+    on Evolutionary Computation, 23(6):972–986, December 2019.
+
 .. [Oszycka1995]
-    A. Osyczka, S. Kundu. 1995. A new method to solve generalized multicriteria
-    optimization problems using the simple genetic algorithm. In Structural
-    Optimization 10. 94–99.
+    A. Osyczka and S. Kundu. A new method to solve generalized
+    multicriteria optimization problems using the simple genetic algorithm.
+    In Structural Optimization 10. 94–99, 1995.
 
 .. [Tanabe2020]
-    Ryoji Tanabe, Hisao Ishibuchi, An easy-to-use real-world multi-objective
+    Ryoji Tanabe and Hisao Ishibuchi. An easy-to-use real-world multi-objective
     optimization problem suite, Applied Soft Computing,Volume 89, 2020.
 
 .. [Yang2019a]
-    K. Yang, M. Emmerich, A. Deutz, and T. Bäck. 2019.
-    "Multi-Objective Bayesian Global Optimization using expected hypervolume
-    improvement gradient" in Swarm and evolutionary computation 44, pp. 945--956,
-    2019.
+    K. Yang, M. Emmerich, A. Deutz, and T. Bäck. Multi-Objective Bayesian
+    Global Optimization using expected hypervolume improvement gradient.
+    Swarm and evolutionary computation 44, pp. 945--956, 2019.
 
 .. [Zitzler2000]
-    E. Zitzler, K. Deb, and L. Thiele, “Comparison of multiobjective
-    evolutionary algorithms: Empirical results,” Evol. Comput., vol. 8, no. 2,
-    pp. 173–195, 2000.
+    E. Zitzler, K. Deb, and L. Thiele. Comparison of multiobjective
+    evolutionary algorithms: Empirical results. Evolutionary Computation, vol.
+    8, no. 2,pp. 173–195, 2000.
 """
 
 from __future__ import annotations
 
 import math
 from abc import ABC, abstractmethod
+from math import pi
 from typing import Optional
 
 import torch
+from botorch.exceptions.errors import UnsupportedError
 from botorch.test_functions.base import (
     ConstrainedBaseTestProblem,
     MultiObjectiveTestProblem,
 )
-from botorch.test_functions.synthetic import Branin
+from botorch.test_functions.synthetic import Branin, Levy
 from botorch.utils.sampling import sample_hypersphere, sample_simplex
 from botorch.utils.transforms import unnormalize
 from scipy.special import gamma
 from torch import Tensor
+from torch.distributions import MultivariateNormal
 
 
 class BraninCurrin(MultiObjectiveTestProblem):
@@ -551,6 +572,278 @@ class DTLZ7(DTLZ):
         return torch.cat([f, ((1 + g_X) * h).unsqueeze(-1)], dim=-1)
 
 
+class GMM(MultiObjectiveTestProblem):
+    r"""A test problem where each objective is a Gaussian mixture model.
+
+    This implementation is adapted from the single objective version (proposed by
+    [Frohlich2020]_) at
+    https://github.com/boschresearch/NoisyInputEntropySearch/blob/master/
+    core/util/objectives.py.
+
+    See [Daulton2022]_ for details on this multi-objective problem.
+    """
+    dim = 2
+    _bounds = [(0.0, 1.0), (0.0, 1.0)]
+
+    def __init__(
+        self,
+        noise_std: Optional[float] = None,
+        negate: bool = False,
+        num_objectives: int = 2,
+    ) -> None:
+        r"""Constructor.
+
+        Args:
+            noise_std: Standard deviation of the observation noise.
+            negate: If True, negate the objectives.
+            num_objectives: The number of objectives.
+        """
+        if num_objectives not in (2, 3, 4):
+            raise UnsupportedError("GMM only currently supports 2 to 4 objectives.")
+        self._ref_point = [-0.2338, -0.2211]
+        if num_objectives > 2:
+            self._ref_point.append(-0.5180)
+        if num_objectives > 3:
+            self._ref_point.append(-0.1866)
+        self.num_objectives = num_objectives
+        super().__init__(noise_std=noise_std, negate=negate)
+        gmm_pos = torch.tensor(
+            [
+                [[0.2, 0.2], [0.8, 0.2], [0.5, 0.7]],
+                [[0.07, 0.2], [0.4, 0.8], [0.85, 0.1]],
+            ]
+        )
+        gmm_var = torch.tensor([[0.20, 0.10, 0.10], [0.2, 0.1, 0.05]]).pow(2)
+        gmm_norm = 2 * pi * gmm_var * torch.tensor([0.5, 0.7, 0.7])
+        if num_objectives > 2:
+            gmm_pos = torch.cat(
+                [gmm_pos, torch.tensor([[[0.08, 0.21], [0.45, 0.75], [0.86, 0.11]]])],
+                dim=0,
+            )
+            gmm_var = torch.cat(
+                [gmm_var, torch.tensor([[0.2, 0.1, 0.07]]).pow(2)], dim=0
+            )
+            gmm_norm = torch.cat(
+                [
+                    gmm_norm,
+                    2 * pi * gmm_var[2] * torch.tensor([[0.5, 0.7, 0.9]]),
+                ],
+                dim=0,
+            )
+        if num_objectives > 3:
+            gmm_pos = torch.cat(
+                [gmm_pos, torch.tensor([[[0.09, 0.19], [0.44, 0.72], [0.89, 0.13]]])],
+                dim=0,
+            )
+            gmm_var = torch.cat(
+                [gmm_var, torch.tensor([[0.15, 0.07, 0.09]]).pow(2)], dim=0
+            )
+            gmm_norm = torch.cat(
+                [
+                    gmm_norm,
+                    2 * pi * gmm_var[3] * torch.tensor([[0.5, 0.7, 0.9]]),
+                ],
+                dim=0,
+            )
+        gmm_covar = gmm_var.view(*gmm_var.shape, 1, 1) * torch.eye(
+            2, dtype=gmm_var.dtype, device=gmm_var.device
+        )
+        self.register_buffer("gmm_pos", gmm_pos)
+        self.register_buffer("gmm_covar", gmm_covar)
+        self.register_buffer("gmm_norm", gmm_norm)
+
+    def evaluate_true(self, X: Tensor) -> Tensor:
+        r"""Evaluate the GMMs."""
+        # This needs to be reinstantiated because MVN apparently does not
+        # have a `to` method to make it device/dtype agnostic.
+        mvn = MultivariateNormal(loc=self.gmm_pos, covariance_matrix=self.gmm_covar)
+        view_shape = (
+            X.shape[:-1]
+            + torch.Size([1] * (self.gmm_pos.ndim - 1))
+            + self.gmm_pos.shape[-1:]
+        )
+        expand_shape = X.shape[:-1] + self.gmm_pos.shape
+        pdf_X = mvn.log_prob(X.view(view_shape).expand(expand_shape)).exp()
+        # Multiply by -1 to make this a minimization problem by default
+        return -(self.gmm_norm * pdf_X).sum(dim=-1)
+
+
+class Penicillin(MultiObjectiveTestProblem):
+    r"""A penicillin production simulator from [Liang2021]_.
+
+    This implementation is adapted from
+    https://github.com/HarryQL/TuRBO-Penicillin.
+
+    The goal is to maximize the penicillin yield while minimizing
+    time to ferment and the CO2 byproduct.
+
+    The function is defined for minimization of all objectives.
+
+    The reference point was set using the `infer_reference_point` heuristic
+    on the Pareto frontier over a large discrete set of random designs.
+    """
+    dim = 7
+    num_objectives = 3
+    _bounds = [
+        (60.0, 120.0),
+        (0.05, 18.0),
+        (293.0, 303.0),
+        (0.05, 18.0),
+        (0.01, 0.5),
+        (500.0, 700.0),
+        (5.0, 6.5),
+    ]
+    _ref_point = [1.85, 86.93, 514.70]
+
+    Y_xs = 0.45
+    Y_ps = 0.90
+    K_1 = 10 ** (-10)
+    K_2 = 7 * 10 ** (-5)
+    m_X = 0.014
+    alpha_1 = 0.143
+    alpha_2 = 4 * 10 ** (-7)
+    alpha_3 = 10 ** (-4)
+    mu_X = 0.092
+    K_X = 0.15
+    mu_p = 0.005
+    K_p = 0.0002
+    K_I = 0.10
+    K = 0.04
+    k_g = 7.0 * 10 ** 3
+    E_g = 5100.0
+    k_d = 10.0 ** 33
+    E_d = 50000.0
+    lambd = 2.5 * 10 ** (-4)
+    T_v = 273.0  # Kelvin
+    T_o = 373.0
+    R = 1.9872  # CAL/(MOL K)
+    V_max = 180.0
+
+    @classmethod
+    def penicillin_vectorized(cls, X_input: Tensor) -> Tensor:
+        r"""Penicillin simulator, simplified and vectorized.
+
+        The 7 input parameters are (in order): culture volume, biomass
+        concentration, temperature, glucose concentration, substrate feed
+        rate, substrate feed concentration, and H+ concentration.
+
+        Args:
+            X_input: A `n x 7`-dim tensor of inputs.
+
+        Returns:
+            An `n x 3`-dim tensor of (negative) penicillin yield, CO2 and time.
+        """
+        V, X, T, S, F, s_f, H_ = torch.split(X_input, 1, -1)
+        P, CO2 = torch.zeros_like(V), torch.zeros_like(V)
+        H = torch.full_like(H_, 10.0).pow(-H_)
+
+        active = torch.ones_like(V).bool()
+        t_tensor = torch.full_like(V, 2500)
+
+        for t in range(1, 2501):
+            if active.sum() == 0:
+                break
+            F_loss = (
+                V[active]
+                * cls.lambd
+                * (torch.exp(5 * ((T[active] - cls.T_o) / (cls.T_v - cls.T_o))) - 1)
+            )
+            dV_dt = F[active] - F_loss
+            mu = (
+                (cls.mu_X / (1 + cls.K_1 / H[active] + H[active] / cls.K_2))
+                * (S[active] / (cls.K_X * X[active] + S[active]))
+                * (
+                    (cls.k_g * torch.exp(-cls.E_g / (cls.R * T[active])))
+                    - (cls.k_d * torch.exp(-cls.E_d / (cls.R * T[active])))
+                )
+            )
+            dX_dt = mu * X[active] - (X[active] / V[active]) * dV_dt
+            mu_pp = cls.mu_p * (
+                S[active] / (cls.K_p + S[active] + S[active].pow(2) / cls.K_I)
+            )
+            dS_dt = (
+                -(mu / cls.Y_xs) * X[active]
+                - (mu_pp / cls.Y_ps) * X[active]
+                - cls.m_X * X[active]
+                + F[active] * s_f[active] / V[active]
+                - (S[active] / V[active]) * dV_dt
+            )
+            dP_dt = (
+                (mu_pp * X[active])
+                - cls.K * P[active]
+                - (P[active] / V[active]) * dV_dt
+            )
+            dCO2_dt = cls.alpha_1 * dX_dt + cls.alpha_2 * X[active] + cls.alpha_3
+
+            # UPDATE
+            P[active] = P[active] + dP_dt  # Penicillin concentration
+            V[active] = V[active] + dV_dt  # Culture medium volume
+            X[active] = X[active] + dX_dt  # Biomass concentration
+            S[active] = S[active] + dS_dt  # Glucose concentration
+            CO2[active] = CO2[active] + dCO2_dt  # CO2 concentration
+
+            # Update active indices
+            full_dpdt = torch.ones_like(P)
+            full_dpdt[active] = dP_dt
+            inactive = (V > cls.V_max) + (S < 0) + (full_dpdt < 10e-12)
+            t_tensor[inactive] = torch.minimum(
+                t_tensor[inactive], torch.full_like(t_tensor[inactive], t)
+            )
+            active[inactive] = 0
+
+        return torch.stack([-P, CO2, t_tensor], dim=-1)
+
+    def evaluate_true(self, X: Tensor) -> Tensor:
+        # This uses in-place operations. Hence, the clone is to avoid modifying
+        # the original X in-place.
+        return self.penicillin_vectorized(X.view(-1, self.dim).clone()).view(
+            *X.shape[:-1], self.num_objectives
+        )
+
+
+class ToyRobust(MultiObjectiveTestProblem):
+    r"""A 1D problem where the Pareto frontier is sensitive to input noise.
+
+    Specifically, the pareto frontier over the nominal objectives is
+    sensitive to input noise. The first objective is a mixture of a linear
+    function and a sinusoidal function, and the second objective is a modified
+    Levy function, where the second parameter is fixed.
+
+    This function comes from [Daulton2022]_.
+
+    The reference point was set using the `infer_reference_point`
+    heuristic on the Pareto frontier over a large discrete set of
+    random designs.
+    """
+    dim = 1
+    _bounds = [(0.0, 0.7)]
+    _ref_point = [-6.1397, -8.1942]
+    num_objectives = 2
+    levy = Levy()
+
+    def f_1(self, X: Tensor) -> Tensor:
+        p1 = 2.4 - 10 * X - 0.1 * X.pow(2)
+        p2 = 2 * X - 0.1 * X.pow(2)
+        smoother = (X - 0.5).pow(2) + torch.sin(30 * X) * 0.1
+        x_mask = torch.sigmoid((0.2 - X) / 0.005)
+        return -(p1 * x_mask + p2 * (1 - x_mask) + smoother) * 30 + 30
+
+    def f_2(self, X: Tensor) -> Tensor:
+        X = torch.cat(
+            [X, torch.zeros_like(X)],
+            dim=-1,
+        )
+        # Cut out the first part of the function.
+        X = X * 0.95 + 0.03
+        X = unnormalize(X, self.levy.bounds.to(X))
+        Y = self.levy(X).unsqueeze(-1)
+        Y -= X[..., :1].pow(2) * 0.75
+        return Y
+
+    def evaluate_true(self, X: Tensor) -> Tensor:
+        return -torch.cat([self.f_1(X), self.f_2(X)], dim=-1)
+
+
 class VehicleSafety(MultiObjectiveTestProblem):
     r"""Optimize Vehicle crash-worthiness.
 
@@ -884,30 +1177,6 @@ class BNH(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
         return torch.stack([c1, c2], dim=-1)
 
 
-class SRN(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
-    r"""The constrained SRN problem.
-
-    See [GarridoMerchan2020]_ for more details on this problem. Note that this is a
-    minimization problem.
-    """
-
-    dim = 2
-    num_objectives = 2
-    num_constraints = 2
-    _bounds = [(-20.0, 20.0), (-20.0, 20.0)]
-    _ref_point = [0.0, 0.0]  # TODO: Determine proper reference point
-
-    def evaluate_true(self, X: Tensor) -> Tensor:
-        obj1 = 2.0 + ((X - 2.0) ** 2).sum(dim=-1)
-        obj2 = 9.0 * X[..., 0] - (X[..., 1] - 1.0) ** 2
-        return torch.stack([obj1, obj2], dim=-1)
-
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
-        c1 = 225.0 - ((X ** 2) ** 2).sum(dim=-1)
-        c2 = -10.0 - X[..., 0] + 3 * X[..., 1]
-        return torch.stack([c1, c2], dim=-1)
-
-
 class CONSTR(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     r"""The constrained CONSTR problem.
 
@@ -982,6 +1251,101 @@ class C2DTLZ2(DTLZ2, ConstrainedBaseTestProblem):
         return -torch.min(min1, min2).unsqueeze(-1)
 
 
+class DiscBrake(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
+    r"""The Disc Brake problem.
+
+    There are 2 objectives and 4 constraints.
+
+    Both objectives should be minimized.
+
+    See [Tanabe2020]_ for details.
+
+    The reference point was set using the `infer_reference_point`
+    heuristic on the Pareto frontier over a large discrete set of
+    random designs.
+    """
+
+    dim = 4
+    num_objectives = 2
+    num_constraints = 4
+    _bounds = [(55.0, 80.0), (75.0, 110.0), (1000.0, 3000.0), (11.0, 20.0)]
+    _ref_point = [5.7771, 3.9651]
+
+    def evaluate_true(self, X: Tensor) -> Tensor:
+        f = torch.zeros(
+            *X.shape[:-1], self.num_objectives, dtype=X.dtype, device=X.device
+        )
+
+        X1, X2, X3, X4 = torch.split(X, 1, -1)
+        sq_diff = X2.pow(2) - X1.pow(2)
+        f[..., :1] = 4.9 * 1e-5 * sq_diff * (X4 - 1.0)
+        f[..., 1:] = (9.82 * 1e6) * sq_diff / (X3 * X4 * (X2.pow(3) - X1.pow(3)))
+
+        return f
+
+    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+        g = torch.zeros(
+            *X.shape[:-1], self.num_constraints, dtype=X.dtype, device=X.device
+        )
+        X1, X2, X3, X4 = torch.split(X, 1, -1)
+        sq_diff = X2.pow(2) - X1.pow(2)
+        cub_diff = X2.pow(3) - X1.pow(3)
+        g[..., :1] = X2 - X1 - 20.0
+        g[..., 1:2] = 0.4 - X3 / (3.14 * sq_diff)
+        g[..., 2:3] = 1.0 - (2.22 * 1e-3 * X3 * cub_diff) / sq_diff.pow(2)
+        g[..., 3:] = (2.66 * 1e-2 * X3 * X4 * cub_diff) / sq_diff - 900.0
+        return g
+
+
+class MW7(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
+    r"""The MW7 problem.
+
+    This problem has 2 objectives, 2 constraints, and a disconnected Pareto
+    frontier. It supports arbitrary input dimension > 1. See [Ma2019]_ for details.
+
+    This implementation is adapted from:
+    https://github.com/anyoptimization/pymoo/blob/master/pymoo/problems/multi/mw.py
+    """
+    num_constraints = 2
+    num_objectives = 2
+    _ref_point = [1.2, 1.2]
+
+    def __init__(
+        self,
+        dim: int,
+        noise_std: Optional[float] = None,
+        negate: bool = False,
+    ) -> None:
+        if dim < 2:
+            raise ValueError("dim must be greater than or equal to 2.")
+        self.dim = dim
+        self._bounds = [(0.0, 1.0) for _ in range(self.dim)]
+        super().__init__(noise_std=noise_std, negate=negate)
+
+    def LA2(self, A, B, C, D, theta):
+        return A * torch.sin(B * theta.pow(C)).pow(D)
+
+    def evaluate_true(self, X: Tensor) -> Tensor:
+        a = X[..., :-1] - 0.5
+        contrib = 2 * (X[..., 1:] + a.pow(2) - 1).pow(2)
+        g = 1 + contrib.sum(dim=-1)
+        f0 = g * X[..., 0]
+        f1 = g * torch.sqrt(1 - (f0 / g).pow(2))
+        return torch.stack([f0, f1], dim=-1)
+
+    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+        ff = self.evaluate_true(X)
+        f0, f1 = ff[..., 0], ff[..., 1]
+        atan = torch.arctan(f1 / f0)
+        g0 = (
+            f0.pow(2)
+            + f1.pow(2)
+            - (1.2 + (self.LA2(0.4, 4.0, 1.0, 16.0, atan)).abs()).pow(2)
+        )
+        g1 = (1.15 - self.LA2(0.2, 4.0, 1.0, 8.0, atan)).pow(2) - f0.pow(2) - f1.pow(2)
+        return -torch.stack([g0, g1], dim=-1)
+
+
 class OSY(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     r"""
     The OSY test problem from [Oszycka1995]_.
@@ -1024,6 +1388,30 @@ class OSY(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
         return torch.stack([g1, g2, g3, g4, g5, g6], dim=-1)
 
 
+class SRN(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
+    r"""The constrained SRN problem.
+
+    See [GarridoMerchan2020]_ for more details on this problem. Note that this is a
+    minimization problem.
+    """
+
+    dim = 2
+    num_objectives = 2
+    num_constraints = 2
+    _bounds = [(-20.0, 20.0), (-20.0, 20.0)]
+    _ref_point = [0.0, 0.0]  # TODO: Determine proper reference point
+
+    def evaluate_true(self, X: Tensor) -> Tensor:
+        obj1 = 2.0 + ((X - 2.0) ** 2).sum(dim=-1)
+        obj2 = 9.0 * X[..., 0] - (X[..., 1] - 1.0) ** 2
+        return torch.stack([obj1, obj2], dim=-1)
+
+    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+        c1 = 225.0 - ((X ** 2) ** 2).sum(dim=-1)
+        c2 = -10.0 - X[..., 0] + 3 * X[..., 1]
+        return torch.stack([c1, c2], dim=-1)
+
+
 class WeldedBeam(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     r"""
     The Welded Beam test problem.
@@ -1075,4 +1463,4 @@ class WeldedBeam(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
         g2 = (1 / s_max) * (s - s_max)
         g3 = (1 / (5 - 0.125)) * (X[..., 0] - X[..., 3])
         g4 = (1 / P) * (P - P_c)
-        return torch.stack([g1, g2, g3, g4], dim=-1)
+        return -torch.stack([g1, g2, g3, g4], dim=-1)
